@@ -107,7 +107,7 @@ impl Helper {
                 .for_storage_encryption().for_transport_encryption()
             {
                 let pair = KeyPair::new_for_gnupg_context(ctx, ka.key())?
-                    .with_cert(ka.cert());
+                    .with_cert(ka.valid_cert());
                 keys.insert(ka.key().keyid(), pair);
             }
         }
@@ -117,17 +117,19 @@ impl Helper {
 }
 
 impl DecryptionHelper for Helper {
-    fn decrypt<D>(&mut self,
-                  pkesks: &[openpgp::packet::PKESK],
-                  _skesks: &[openpgp::packet::SKESK],
-                  sym_algo: Option<SymmetricAlgorithm>,
-                  mut decrypt: D)
-                  -> openpgp::Result<Option<openpgp::Fingerprint>>
-        where D: FnMut(SymmetricAlgorithm, &SessionKey) -> bool
+    fn decrypt(&mut self,
+               pkesks: &[openpgp::packet::PKESK],
+               _skesks: &[openpgp::packet::SKESK],
+               sym_algo: Option<SymmetricAlgorithm>,
+               decrypt: &mut dyn FnMut(Option<SymmetricAlgorithm>, &SessionKey) -> bool)
+               -> openpgp::Result<Option<openpgp::Cert>>
     {
         // Try each PKESK until we succeed.
         for pkesk in pkesks {
-            if let Some(pair) = self.keys.get_mut(pkesk.recipient()) {
+            if let Some(pair) = self.keys.get_mut(
+                &pkesk.recipient().map(openpgp::KeyID::from)
+                    .unwrap_or_else(openpgp::KeyID::wildcard))
+            {
                 if pkesk.decrypt(pair, sym_algo)
                     .map(|(algo, session_key)| decrypt(algo, &session_key))
                     .unwrap_or(false)
@@ -188,6 +190,12 @@ impl VerificationHelper for Helper {
                             Err(BadSignature { error, .. }) => {
                                 eprintln!("Verifying signature: {}.", error);
                             },
+                            Err(UnknownSignature { sig, .. }) => {
+                                eprintln!("Verifying signature: {}.", sig.error());
+                            },
+                            Err(e) => {
+                                eprintln!("Verifying signature: {}.", e);
+                            },
                         }
                     }
             }
