@@ -68,7 +68,7 @@ fn main() -> openpgp::Result<()> {
         cert.keys().with_policy(p, None).alive().revoked(false).for_signing()
             .filter_map(|ka| {
                 KeyPair::new_for_gnupg_context(&ctx, ka.key())
-                    .map(|kp| kp.with_cert(ka.cert()))
+                    .map(|kp| kp.with_cert(ka.valid_cert()))
                     .ok()
             })
     }).collect::<Vec<KeyPair>>();
@@ -84,9 +84,9 @@ fn main() -> openpgp::Result<()> {
 
     // Now, create a signer that emits the signature(s).
     let mut signer =
-        Signer::new(message, signers.pop().expect("No key for signing"));
+        Signer::new(message, signers.pop().expect("No key for signing"))?;
     for s in signers {
-        signer = signer.add_signer(s);
+        signer = signer.add_signer(s)?;
     }
     let signer = signer.build()?;
 
