@@ -172,8 +172,8 @@ impl Client {
     ///
     /// Note: `command` is passed as-is.  Control characters, like
     /// `%`, must be %-escaped using [`escape`].
-    pub fn send<'a, C: 'a>(&'a mut self, command: C) -> Result<()>
-        where C: AsRef<[u8]>
+    pub fn send<'a, C>(&'a mut self, command: C) -> Result<()>
+        where C: AsRef<[u8]> + 'a
     {
         if let WriteState::Sending(_) = self.w {
             return Err(Error::InvalidOperation(
@@ -268,8 +268,8 @@ impl Client {
     /// [`Response::Ok`]: super::assuan::Response::Ok
     /// [`Response::Error`]: super::assuan::Response::Error
     /// [`Response::Inquire`]: super::assuan::Response::Inquire
-    pub fn data<'a, C: 'a>(&'a mut self, data: C) -> Result<()>
-        where C: AsRef<[u8]>
+    pub fn data<'a, C>(&'a mut self, data: C) -> Result<()>
+        where C: AsRef<[u8]> + 'a
     {
         let mut data = data.as_ref();
         let mut request = Vec::with_capacity(data.len());
