@@ -39,6 +39,9 @@ macro_rules! make_context {
             },
         };
 
+        // To debug gpg-agent, launch `watchgnupg --force /tmp/S.log`
+        // in a terminal and add `log-file socket:///tmp/S.log\n` to
+        // the configuration here:
         std::fs::write(ctx.homedir().unwrap().join("gpg-agent.conf"),
                        "allow-loopback-pinentry\n").unwrap();
 
@@ -140,6 +143,7 @@ fn sign() -> openpgp::Result<()> {
 
       for password in vec![None, Some(PASSWORD.into())] {
         let (cert, _) = CertBuilder::new()
+            .set_profile(openpgp::Profile::RFC4880)?
             .set_cipher_suite(*cs)
             .add_userid("someone@example.org")
             .add_signing_subkey()
@@ -166,7 +170,7 @@ fn sign() -> openpgp::Result<()> {
             let message = Message::new(&mut message);
 
             // We want to sign a literal data packet.
-            let signer = Signer::new(message, keypair)
+            let signer = Signer::new(message, keypair)?
                  // XXX: Is this necessary?  If so, it shouldn't.
                 .hash_algo(HashAlgorithm::SHA512).unwrap()
                 .build().unwrap();
@@ -279,6 +283,7 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
         let ctx = make_context!();
 
         let (cert, _) = CertBuilder::new()
+            .set_profile(openpgp::Profile::RFC4880)?
             .set_cipher_suite(*cs)
             .add_userid("someone@example.org")
             .add_transport_encryption_subkey()
@@ -299,8 +304,7 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
             let recipients =
                 [&cert, &other].iter().flat_map(
                     |c| c.keys().with_policy(p, None).alive().revoked(false)
-                        .for_transport_encryption()
-                        .map(|ka| ka.key()))
+                        .for_transport_encryption())
                 .collect::<Vec<_>>();
 
             // Start streaming an OpenPGP message.
@@ -308,7 +312,7 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
 
             // We want to encrypt a literal data packet.
             let encryptor =
-                Encryptor2::for_recipients(message, recipients)
+                Encryptor::for_recipients(message, recipients)
                 .build().unwrap();
 
             // Emit a literal data packet.
@@ -398,13 +402,12 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
         }
 
         impl<'a> DecryptionHelper for Helper<'a> {
-            fn decrypt<D>(&mut self,
-                          pkesks: &[openpgp::packet::PKESK],
-                          _skesks: &[openpgp::packet::SKESK],
-                          sym_algo: Option<SymmetricAlgorithm>,
-                          mut decrypt: D)
-                          -> openpgp::Result<Option<openpgp::Fingerprint>>
-                where D: FnMut(SymmetricAlgorithm, &SessionKey) -> bool
+            fn decrypt(&mut self,
+                       pkesks: &[openpgp::packet::PKESK],
+                       _skesks: &[openpgp::packet::SKESK],
+                       sym_algo: Option<SymmetricAlgorithm>,
+                       decrypt: &mut dyn FnMut(Option<SymmetricAlgorithm>, &SessionKey) -> bool)
+                       -> openpgp::Result<Option<openpgp::Cert>>
             {
                 // We only gave the cert to GnuPG, the agent doesn't
                 // have the secret.
@@ -432,7 +435,11 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
                 }
 
                 for pkesk in pkesks {
-                    if *pkesk.recipient() != keypair.public().keyid() {
+                    let key_handle = keypair.public().key_handle();
+                    if pkesk.recipient()
+                        .map(|r| ! r.aliases(&key_handle))
+                        .unwrap_or(false)
+                    {
                         continue;
                     }
 
@@ -442,8 +449,8 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
                     assert!(decrypt(algo, &session_key));
                 }
 
-                // XXX: In production code, return the Fingerprint of the
-                // recipient's Cert here
+                // XXX: In production code, return the recipient's
+                // Cert here.
                 Ok(None)
             }
         }
@@ -451,3 +458,197 @@ fn decrypt(also_try_explicit_async: bool) -> openpgp::Result<()> {
     }
     Ok(())
 }
+
+/// Tests importing various certificate types.
+#[test]
+fn import() -> openpgp::Result<()> {
+    let rt = tokio::runtime::Runtime::new()?;
+    rt.block_on(import_async())
+}
+
+async fn import_async() -> openpgp::Result<()> {
+    use self::CipherSuite::*;
+    use openpgp::policy::StandardPolicy as P;
+
+    let p = &P::new();
+    let ctx = make_context!();
+
+    for cs in &[RSA2k, RSA3k, RSA4k, Cv25519, P256, P384, P521] {
+        if let Err(_) = cs.is_supported() {
+            continue;
+        }
+        for password in vec![None, Some(PASSWORD.into())] {
+            eprintln!("{:?} with password {:?}", cs, password);
+
+            let (cert, _) = CertBuilder::new()
+                .set_profile(openpgp::Profile::RFC4880)?
+                .set_cipher_suite(*cs)
+                .add_userid("someone@example.org")
+                .add_signing_subkey()
+                .set_password(password.clone())
+                .generate().unwrap();
+
+            let mut agent = Agent::connect(&ctx).await?;
+            for ka in cert.keys() {
+                agent.import(p, &cert, ka.key().parts_as_secret()?, true, true)
+                    .await?;
+            }
+
+            let mut keypair = KeyPair::new_for_gnupg_context(
+                &ctx,
+                cert.keys().with_policy(p, None).alive().revoked(false)
+                    .for_signing().take(1).next().unwrap().key())
+                .unwrap();
+
+            if let Some(p) = password.clone() {
+                keypair = keypair.with_password(p);
+            }
+
+            let mut message = Vec::new();
+            {
+                // Start streaming an OpenPGP message.
+                let message = Message::new(&mut message);
+
+                // We want to sign a literal data packet.
+                let signer = Signer::new(message, keypair)?
+                // XXX: Is this necessary?  If so, it shouldn't.
+                    .hash_algo(HashAlgorithm::SHA512).unwrap()
+                    .build().unwrap();
+
+                // Emit a literal data packet.
+                let mut literal_writer = LiteralWriter::new(
+                    signer).build().unwrap();
+
+                // Sign the data.
+                literal_writer.write_all(MESSAGE.as_bytes()).unwrap();
+
+                // Finalize the OpenPGP message to make sure that all data is
+                // written.
+                literal_writer.finalize().unwrap();
+            }
+
+            // Make a helper that that feeds the sender's public key to the
+            // verifier.
+            let helper = Helper { cert: &cert };
+
+            // Now, create a verifier with a helper using the given Certs.
+            let mut verifier = VerifierBuilder::from_bytes(&message)?
+                .with_policy(p, None, helper)?;
+
+            // Verify the data.
+            let mut sink = Vec::new();
+            io::copy(&mut verifier, &mut sink).unwrap();
+            assert_eq!(MESSAGE.as_bytes(), &sink[..]);
+        }
+
+        struct Helper<'a> {
+            cert: &'a openpgp::Cert,
+        }
+
+        impl<'a> VerificationHelper for Helper<'a> {
+            fn get_certs(&mut self, _ids: &[openpgp::KeyHandle])
+                         -> openpgp::Result<Vec<openpgp::Cert>> {
+                // Return public keys for signature verification here.
+                Ok(vec![self.cert.clone()])
+            }
+
+            fn check(&mut self, structure: MessageStructure)
+                     -> openpgp::Result<()> {
+                // In this function, we implement our signature verification
+                // policy.
+
+                let mut good = false;
+                for (i, layer) in structure.into_iter().enumerate() {
+                    match (i, layer) {
+                        // First, we are interested in signatures over the
+                        // data, i.e. level 0 signatures.
+                        (0, MessageLayer::SignatureGroup { results }) => {
+                            // Finally, given a VerificationResult, which only says
+                            // whether the signature checks out mathematically, we apply
+                            // our policy.
+                            match results.into_iter().next() {
+                                Some(Ok(_)) => good = true,
+                                Some(Err(e)) =>
+                                    return Err(openpgp::Error::from(e).into()),
+                                None => (),
+                            }
+                        },
+                        _ => return Err(anyhow::anyhow!(
+                            "Unexpected message structure")),
+                    }
+                }
+
+                if good {
+                    Ok(()) // Good signature.
+                } else {
+                    Err(anyhow::anyhow!("Signature verification failed"))
+                }
+            }
+        }
+    }
+    Ok(())
+}
+
+/// Tests importing increasingly short Ed25519 secret keys.
+#[test]
+fn import_ecc_scalars() -> openpgp::Result<()> {
+    let rt = tokio::runtime::Runtime::new()?;
+    rt.block_on(import_ecc_scalars_async())
+}
+
+async fn import_ecc_scalars_async() -> openpgp::Result<()> {
+    use openpgp::policy::StandardPolicy as P;
+
+    let p = &P::new();
+    let ctx = make_context!();
+
+    let mut scalar = [0; 32];
+    while scalar.iter().any(|o| *o == 0) {
+        openpgp::crypto::random(&mut scalar).unwrap();
+    }
+
+    for n in 0..scalar.len() {
+        eprintln!("\n\n{}\n\n", n);
+
+        scalar[..n].iter_mut().for_each(|o| *o = 0);
+
+        use openpgp::packet::key;
+        let key: key::Key<_, key::PrimaryRole> =
+            key::Key4::import_secret_ed25519(&scalar, None).unwrap()
+            .into();
+        let cert = Cert::from_packets(vec![key.into()].into_iter()).unwrap();
+
+        let mut agent = Agent::connect(&ctx).await.unwrap();
+        for ka in cert.keys() {
+            agent.import(p, &cert, ka.key().parts_as_secret().unwrap(), true, true)
+                .await.unwrap();
+        }
+
+        let keypair = KeyPair::new_for_gnupg_context(
+            &ctx, cert.keys().next().unwrap().key()).unwrap();
+
+        let mut message = Vec::new();
+
+        // Start streaming an OpenPGP message.
+        let message = Message::new(&mut message);
+
+        // We want to sign a literal data packet.
+        let signer = Signer::new(message, keypair).unwrap()
+        // XXX: Is this necessary.unwrap()  If so, it shouldn't.
+            .hash_algo(HashAlgorithm::SHA512).unwrap()
+            .build().unwrap();
+
+        // Emit a literal data packet.
+        let mut literal_writer = LiteralWriter::new(
+            signer).build().unwrap();
+
+        // Sign the data.
+        literal_writer.write_all(MESSAGE.as_bytes()).unwrap();
+
+        // Finalize the OpenPGP message to make sure that all data is
+        // written.
+        literal_writer.finalize().unwrap();
+    }
+
+    Ok(())
+}
